
## remove (almost) everything in the working environment.
rm(list = ls())

# to import xl files
library(readxl)
#to write excel files
require(writexl)
# to output towards latex
require(stargazer)


setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\base")


#=========================================================================================

#load database of families
families <- as.data.frame(read_excel("Families-bis.xlsx"))
nfamil<-nrow(families)



#Opening the dictionary containing notoriety metrics for each individual
basew <- as.data.frame(read_excel("Dictionary.xlsx"))


#Opening the base containing other infos for each individual
bigbase <- as.data.frame(read_excel("Profs.xlsx"))
bigbase$refdatel <- pmin(bigbase$Beginning,bigbase$DateApprox,
         bigbase$YearBorn+30,bigbase$End,bigbase$YearDied,na.rm=T)

#bigbase <- bigbase[!is.na(bigbase$refdatel),]

#Opening the base containing other infos for each institution
institutions <- as.data.frame(read_excel("Institutions-All.xlsx"))


#compute number of families by institution in a new dataframe
ilist=unique(families$IKey)
instit <- data.frame("IKey" = ilist, "Nb" = 0 ,"Youngest" =0,"Oldest"=0,"Univ"=0)
for (i in ilist){
instit[instit$IKey==i,"Nb"] <- nrow(families[families$IKey==i,])
instit[instit$IKey==i,"Univ"] <- institutions[institutions$IKey==i,"University"]
}

#here we change the assumption: no page wc implies zero publications (not one as in the database)
basew$nworks <- ifelse(basew$pagewc==1,basew$nworks,0)
basew$nlib <- ifelse(basew$pagewc==1,basew$nlib,0)

families$Sworks <- 0  #number of works by the son
families$Fworks <- 0  #number of works by the father
families$Sin <- 0  #whether son is in WC
families$Fin <- 0  #whether father is in WC


#collect data from the dictionary
# the data cropped from worldcat include:
#nworks	publications	publi_by	publi_about	nlang	nlib

for (i in 1:nfamil)
{
families[i,"Sworks"] <-  as.numeric(basew[basew$PKey==families[i,"Son"],"nworks"]) 
families[i,"Fworks"] <-  as.numeric(basew[basew$PKey==families[i,"Father"],"nworks"])
families[i,"Slibh"] <-  as.numeric(basew[basew$PKey==families[i,"Son"],"nlib"]) 
families[i,"Flibh"] <-  as.numeric(basew[basew$PKey==families[i,"Father"],"nlib"])
families[i,"Sin"] <-  as.numeric(basew[basew$PKey==families[i,"Son"],"pagewc"]) 
families[i,"Fin"] <-  as.numeric(basew[basew$PKey==families[i,"Father"],"pagewc"])
families[i,"Sinwiki"] <-  as.numeric(basew[basew$PKey==families[i,"Son"],"pagewiki"]) 
families[i,"Finwiki"] <-  as.numeric(basew[basew$PKey==families[i,"Father"],"pagewiki"])
}


families$Sdate <- 0  # reference date for the son (when he enters the population of scholars)
families$Fdate <- 0  #reference date for the father

#collect data from the dictionary
for (i in 1:nfamil)
{
tpson <- bigbase[bigbase$PKey==families[i,"Son"],]
tpfather<-bigbase[bigbase$PKey==families[i,"Father"],]
families[i,"Sdate"] <-  min(tpson$refdatel) 
families[i,"Fdate"] <-  min(tpfather$refdatel)
families[i,"Sdied"] <-  ifelse(is.na(min(tpson$YearDied,na.rm=T)), NA, min(tpson$YearDied))
families[i,"Fdied"] <-  ifelse(is.na(min(tpfather$YearDied,na.rm=T)), NA, min(tpfather$YearDied))
families[i,"Sborn"] <-  ifelse(is.na(min(tpson$YearBorn,na.rm=T)), NA, min(tpson$YearBorn))
families[i,"Fborn"] <-  ifelse(is.na(min(tpfather$YearBorn,na.rm=T)), NA, min(tpfather$YearBorn))
}




#HERE we take the library holdings as a measure of publications
families$Spubli <- asinh(families$Slibh)
families$Fpubli <- asinh(families$Flibh)


reg <- lm (Fpubli~Fdate,data=families[families$Fpubli>0,])
summary(reg)

#display number of cases by presence in WorldCat
table(families$Sin,families$Fin)

cor(families$Sin,families$Fin,method="spearman")

#defining set of families with positive publications
families2 <- families[families$Spubli>0 & families$Fpubli>0,]
cor(families2$Spubli,families2$Fpubli)


#display number of cases by presence in WorldCat vs Wiki

table(families$Sin,families$Sinwiki)
table(families$Fin,families$Finwiki)



#regressions
fit1<-lm(Spubli ~ Fpubli,data=families)
fit2<-lm(Spubli ~ Fpubli,data=families2)

stargazer(fit1,fit2)


#list of sons
list_sons <- unique(families$Son)

#fathers who are themselves sons whom I will eliminate
list_fathers <- families[families$Father %in% list_sons,"Father"]
list_grandsons <-  families[families$Father %in% list_fathers,"Son"]

gfamilies <- data.frame("GrandFather"=character(),"GrandSon"=character(),"GFpubli"=numeric(),"GSpubli"=numeric(), 
                                 "GFin"=numeric(),"GSin"=numeric(),stringsAsFactors=FALSE)

ngfamil<-length(list_grandsons)


for (v in (1:ngfamil)){
grdson <- list_grandsons[v]
gfamilies[v,"GrandSon"]<-grdson
gfamilies[v,"GSpubli"]<-unique(families[families$Son==grdson,"Spubli"])
gfamilies[v,"GSin"]<-unique(families[families$Son==grdson,"Sin"])
son <- unique(families[families$Son==grdson,"Father"])
grdfather <- unique(families[families$Son==son,"Father"])
gfamilies[v,"GrandFather"]<-grdfather
gfamilies[v,"GFpubli"]<-unique(families[families$Father==grdfather,"Fpubli"])
gfamilies[v,"GFin"]<-unique(families[families$Father==grdfather,"Fin"])
}

gfamilies <- gfamilies[!(gfamilies$GrandFather %in% list_sons),]



#list of moments
mom<-numeric(13)
PD<- nrow(families)
mom[1] <- cor(families[!(families$Father %in% list_sons) & families$Fin>0 & families$Sin>0,"Fpubli"],
                               families[!(families$Father %in% list_sons) & families$Fin>0 & families$Sin>0,"Spubli"])
mom[2] <- nrow(families[families$Sin==0,])/PD #density at 0 for sons
mom[3]<- as.numeric(quantile(families$Spubli,0.5)) #median for sons
mom[4]<- as.numeric(quantile(families$Spubli,0.75)) # third quartile for sons
mom[5]<- as.numeric(quantile(families$Spubli,0.95)) #upper tail for sons
mom[6]<-mean(families$Spubli)
mom[7] <- nrow(families[!(families$Father %in% list_sons) & families$Fin==0,])/nrow(families[!(families$Father %in% list_sons),]) #same four moments for fathers
mom[8]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.5))
mom[9]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.75))
mom[10]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.95))
mom[11]<-mean(families[!(families$Father %in% list_sons),"Fpubli"])
mom[12]<- nrow(families[families$Fin==0 & families$Sin==0,])/PD
mom[13] <- cor(gfamilies[gfamilies$GFin>0 & gfamilies$GSin>0,"GFpubli"],
                       gfamilies[gfamilies$GFin>0 & gfamilies$GSin>0,"GSpubli"])

setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\database")
#save what was found
write_xlsx(families,"Families-publis-bis.xlsx")
write_xlsx(gfamilies,"GFamilies-publis-bis.xlsx")





sink("moments-bis.txt")


families.main <- as.data.frame(read_excel("Families-publis.xlsx"))
families.alt <- as.data.frame(read_excel("Families-publis-bis.xlsx"))
list.sons.main<-unique(families.main$Son)
list.sons.alt<-unique(families.alt$Son)
list.sons.inters <- intersect(list.sons.main,list.sons.alt)

print('% of sons in the main sample being in the validation sample (had an occupation elsewhere)')
length(list.sons.inters)/length(list.sons.main)




nf<-nrow(families[!(families.alt$Father %in% list.sons.alt),])
print(c("number of fathers:",nf))

ns<-nrow(families.alt)
print(c("number of sons",ns))

newsons <- (length(list.sons.alt)-length(list.sons.inters))/length(list.sons.alt)
print(c("share of sons not at all in the same institution as fathers",newsons))

print("Cases for which birth date is known")
(nrow(families.alt[!(families.alt$Father %in% list.sons.alt) & !is.na(families.alt$Fborn),])+
   nrow(families.alt[!is.na(families.alt$Sborn),]))/(nf+ns)

print("Cases for which death date is known")
(nrow(families.alt[!(families.alt$Father %in% list.sons.alt) & !is.na(families.alt$Fdied),])+
   nrow(families.alt[!is.na(families.alt$Sdied),]))/(nf+ns)

print("Cases for which nomination date is known")
(nrow(families.alt[!(families.alt$Father %in% list.sons.alt) & !is.na(families.alt$Fstart),])+
   nrow(families.alt[!is.na(families.alt$Sstart),]))/(nf+ns)

labels<- c("corr(F,S)","% S with 0","Q50 S","Q75 S","Q95 S","Mean S","% F with 0",
"Q50 F","Q75 F","Q95 F","Mean F","% S&F with 0","corr(GF,GS)")
cbind(labels,round(mom,3))

nrow(gfamilies)
length(gfamilies[gfamilies$GFin>0 & gfamilies$GSin>0,"GFpubli"])


sink()





