

## remove (almost) everything in the working environment.
rm(list = ls())


# to import xl files
library(readxl)

setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\database")
#load database of families
families <- as.data.frame(read_excel("families-publis.xlsx"))

#defining set of families with positive publications
families2 <- families[families$Spubli>0 & families$Fpubli>0,]
fit2<-lm(Spubli ~ Fpubli,data=families2)

#upper bound for axes
maxp <- max(c(asinh(families$Slibh),asinh(families$Flibh)))



pdf("OLS-beta.pdf", width = 8 , height = 8)

plot((families2$Spubli)~(families2$Fpubli),xlab="",ylab="",
         xlim=c(0, 12),ylim=c(0, 12), main=NULL,cex=.6)
title(ylab="Sons' publications",line=2,cex.lab=1)
title(xlab="Fathers' publications", line=3, cex.lab=1)
abline(coef(fit2)[1:2],lw=2)
with(families, text(families2$Spubli~families2$Fpubli,
 labels =unlist(lapply(strsplit(families2$Father, split = "-"), `[`, 1)), pos = 1,cex=.6))

#https://uc-r.github.io/string_manipulation_baseR/
#https://stackoverflow.com/questions/13016359/how-to-directly-select-the-same-column-from-all-nested-lists-within-a-list

dev.off()




pdf("histogram-fathers.pdf", width = 16 , height = 5)
hist(families$Fdate,xlab="Fathers' reference date",main="")
dev.off()
pdf("publications-over-time.pdf", width = 16 , height = 5)
plot(families$Fpubli~families$Fdate,xlab="Fathers' reference date",
ylab="publications",main="",bty="n")
dev.off()


t1<-ks.test(families$Fpubli,families$Spubli)

pdf("QQ-plot-slides.pdf", width = 8 , height = 8)
qqplot(families$Fpubli,families$Spubli,xlab="",ylab="",yaxt="n",xaxt="n")
title(ylab="Sons' publications", line=1, cex.lab=1.4)
title(xlab="Fathers' publications", line=1, cex.lab=1.4)
abline(a=0,b=1)
dev.off()


###########################QQ : all ######################################################
pdf("QQ-plot-all.pdf", width = 8 , height = 8)

par(mgp=c(2.5,1,0),mar=c(5,4,4,2)+0.1)
qqplot(families$Fpubli,families$Spubli,
xlab="Fathers' publications",
ylab="Sons' publications",xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,cex=.6,lwd=2)
abline(a=0,b=1)
text(1, 12, paste("N.obs=",nrow(families)),cex=.85)
text(4, 11.3, paste("Kolmogorov-Smirnov (Fathers vs Sons) =",round(t1$statistic,3),
  " (p-value=",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9)

dev.off()



###########################QQ : fields ######################################################
pdf("QQ-plot-fields.pdf", width = 8 , height = 8)

rfamilies <-families[families$Field1==2 | (!is.na(families$Field2) & families$Field2==2) |
     (!is.na(families$Field3) & families$Field3==2),]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab="Fathers' publications",ylab="Sons' publications",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=2,col="red",cex=.6,lwd=2)
abline(a=0,b=1)
legend(0, 12.5, legend=paste("Lawyers, n.obs= ",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=2,col="red")

par(new=TRUE)

rfamilies <-families[families$Field1==4 | (!is.na(families$Field2) & families$Field2==4) |
     (!is.na(families$Field3) & families$Field3==4),]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=1,col="blueviolet",cex=.6,lwd=2)
legend(0, 11.5, legend=paste("Physicians, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=1,col="blueviolet")

par(new=TRUE)

rfamilies <-families[families$Field1==1 | (!is.na(families$Field2) & families$Field2==1) |
     (!is.na(families$Field3) & families$Field3==1),]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=3,col="darkred",cex=.6,lwd=2)
legend(0, 10.5, legend=paste("Theologians, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=3,col="darkred")

par(new=TRUE)

rfamilies <-families[families$Field1==5 | (!is.na(families$Field2) & families$Field2==5) |
     (!is.na(families$Field3) & families$Field3==5),]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=4,col="blue",cex=.6,lwd=2)
legend(0, 9.5, legend=paste("Scientists, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=4,col="blue")

dev.off()


###########################QQ : religion ######################################################
pdf("QQ-plot-religion.pdf", width = 8 , height = 8)

rfamilies <-families[families$Protestant==1 & families$Fdate>1527,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab="Fathers' publications",ylab="Sons' publications",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=2,col="gray50",cex=.6,lwd=2)
legend(0, 13, legend=paste("Protestants, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""), pt.lwd=2,cex=0.9,box.lty=0,pch=2,col="gray50")
abline(a=0,b=1)

par(new=TRUE)

rfamilies <-families[families$Catholic==1 & families$Fdate>1527,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=1,col="goldenrod",cex=.6,lwd=2)
legend(0, 12, legend=paste("Catholics aft. 1527, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=1,col="goldenrod")

dev.off()



####removing Italy and Spain
pdf("QQ-plot-religion-without-south.pdf", width = 8 , height = 8)

rfamilies <-families[families$Protestant==1 & families$Fdate>1527 & families$Country!="ITA" & families$Country!="ESP",]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab="Fathers' publications",ylab="Sons' publications",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=2,col="gray50",cex=.6,lwd=2)
legend(0, 13, legend=paste("Protestants, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=2,col="gray50")
abline(a=0,b=1)

par(new=TRUE)

rfamilies <-families[families$Catholic==1 & families$Fdate>1527 & families$Country!="ITA" & families$Country!="ESP",]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=1,col="goldenrod",cex=.6,lwd=2)
legend(0, 12, legend=paste("Catholics aft. 1527 not Italian/Spanish, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,,box.lty=0,pch=1,col="goldenrod")

dev.off()



pdf("publications-religion.pdf", width = 8 , height = 6)

r1families <-families[families$Protestant==1 & families$Fdate>1527,c("Father","Fpubli")]
colnames(r1families)<- c("PKey","publi")
r2families <-families[families$Protestant==1 & families$Fdate>1527,c("Son","Spubli")]
colnames(r2families)<- c("PKey","publi")
rfamilies<- unique(rbind(r1families,r2families))
smpl1<-nrow(rfamilies)

nrow(rfamilies[rfamilies$publi==0,])/smpl1
exp(mean(rfamilies[rfamilies$publi>0,"publi"]))-1

hist(rfamilies$publi,xlim=c(0,14),ylim=c(0,600),
main=" ",col=rgb(0.5,0.5,0.5,2/4),xlab="Publications (in logs)",ylab="Frequency",breaks=30)

r1families <-families[families$Catholic==1 & families$Fdate>1527,c("Father","Fpubli")]
colnames(r1families)<- c("PKey","publi")
r2families <-families[families$Catholic==1 & families$Fdate>1527,c("Son","Spubli")]
colnames(r2families)<- c("PKey","publi")
rfamilies<- unique(rbind(r1families,r2families))
smpl2<-nrow(rfamilies)

exp(mean(rfamilies[rfamilies$publi>0,"publi"]))-1

hist(rfamilies$publi,xlim=c(0,14),ylim=c(0,350),
main="Fathers: obs. publications distribution",col=rgb(218/256,165/256,32/256,2/4),xlab="",ylab="",breaks=30,add=T)
legend(2, 400, legend=paste("members of Protestant institutions, n.obs=",smpl1), cex=1,box.lty=0,pch=15,col="gray50")
legend(2, 450, legend=paste("members of Catholic institutions, n.obs=",smpl2), cex=1,box.lty=0,pch=15,col="goldenrod")

dev.off()
smpl1+smpl2


############################QQ : over time #############################################
#plot over time
pdf("QQ-plot-time.pdf", width = 8 , height = 8)

rfamilies <-families[families$Fdate<1543,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab="Fathers' publications",ylab="Sons' publications",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=1,col="red",cex=.6,lwd=2)
legend(0, 13, legend=paste("Period <1543, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=1,col="red")
abline(a=0,b=1)

par(new=TRUE)

rfamilies <-families[families$Fdate>1542 & families$Fdate<=1632,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=2,col="blueviolet",cex=.6,lwd=2)
legend(0, 12, legend=paste("Sc. Revol. I (1543-1632), n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=2,col="blueviolet")

par(new=TRUE)

rfamilies <-families[families$Fdate>1632 & families$Fdate<=1687,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,
xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=3,col="darkred",cex=.6,lwd=2)
legend(0, 11, legend=paste("Sc. Revol. II (1633-1687), n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=3,col="darkred")

par(new=TRUE)

rfamilies <-families[families$Fdate>1687,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,
xlab=" ",ylab=" ",xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=4,col="blue",cex=.6,lwd=2)
legend(0, 10, legend=paste("Enlightenment (>1687), n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=4,col="blue")

dev.off()



#old vs new#########################
pdf("QQ-plot-new-institutions.pdf", width = 8 , height = 8)

rfamilies <-families[families$New==1 & families$Fdate>1542,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab="Fathers' publications",ylab="Sons' publications",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=2,col="chartreuse2",cex=.6,lwd=2)
legend(0, 13, legend=paste("Families after 1543 in new institutions, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=2,col="chartreuse2")
abline(a=0,b=1)

par(new=TRUE)

rfamilies <-families[families$New==0 & families$Fdate>1542,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=1,col="burlywood4",cex=.6,lwd=2)
legend(0, 12, legend=paste("Families after 1543 in old institutions, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=1,col="burlywood4")

dev.off()





#old vs new##############################################################################
pdf("QQ-plot-type-institutions.pdf", width = 8 , height = 8)

rfamilies <-families[families$University==0 & families$Fdate>1542,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab="Fathers' publications",ylab="Sons' publications",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=2,col="chartreuse3",cex=.6,lwd=2)
legend(0, 13, legend=paste("Families after 1543 in academies, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=2,col="chartreuse3")
abline(a=0,b=1)

par(new=TRUE)

rfamilies <-families[families$University==1 & families$Fdate>1542,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=1,col="lightcoral",cex=.6,lwd=2)
legend(0, 12, legend=paste("Families after 1543 in universities, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=1,col="lightcoral")

dev.off()



#QQ by nomination ##############################################################################


pdf("QQ-plot-nomination.pdf", width = 8 , height = 8)

rfamilies <-families[!is.na(families$Fdied) & !is.na(families$Sstart) & families$Fdied<=families$Sstart,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,
xlab="Fathers' publications",ylab="Sons' publications",
xlim=c(0,maxp),ylim=c(0,maxp),  main=NULL,pch=1,col="red",cex=.6,lwd=2)
legend(0, 13, legend=paste("Sons nominated after father's death, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=1,col="red")
abline(a=0,b=1)

par(new=TRUE)

rfamilies <-families[!is.na(families$Fdied) & !is.na(families$Sstart) & families$Fdied>families$Sstart,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=2,col="gray80",cex=.6,lwd=2)
legend(0, 12, legend=paste("Sons nominated before father's death, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=2,col="gray80")

dev.off()



#QQ by type of institutions##############################################################################


pdf("QQ-plot-univ-vs-acad.pdf", width = 8 , height = 8)

rfamilies <-families[families$University==1 & families$Fdate>1542,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,
xlab="Fathers' publications",ylab="Sons' publications",
xlim=c(0,maxp),ylim=c(0,maxp),  main=NULL,pch=1,col="coral3",cex=.6,lwd=2)
legend(0, 13, legend=paste("Families after 1543 in universities, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=1,col="coral3")
abline(a=0,b=1)

par(new=TRUE)

rfamilies <-families[families$University==0 & families$Fdate>1542,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=2,col="chartreuse4",cex=.6,lwd=2)
legend(0, 12, legend=paste("Families after 1543 in academies, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=2,col="chartreuse4")

dev.off()




#QQ by field of son - same or different than father ##############################################################################


pdf("QQ-plot-same-field.pdf", width = 8 , height = 8)

rfamilies <-families[families$SameField==1,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,
xlab="Fathers' publications",ylab="Sons' publications",
xlim=c(0,maxp),ylim=c(0,maxp),  main=NULL,pch=1,col="coral2",cex=.6,lwd=2)
legend(0, 13, legend=paste("Sons in same field as father, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=1,col="coral2")
abline(a=0,b=1)

par(new=TRUE)

rfamilies <-families[families$SameField==0,]
t1<-ks.test(rfamilies$Fpubli,rfamilies$Spubli)

qqplot(rfamilies$Fpubli,rfamilies$Spubli,xlab=" ",ylab=" ",
xlim=c(0,maxp),ylim=c(0,maxp), main=NULL,pch=2,col="chartreuse3",cex=.6,lwd=2)
legend(0, 12, legend=paste("Sons not is same field as father, n.obs=",nrow(rfamilies),", K-S = ",round(t1$statistic,3),
  " (",round(t1$p.value,3),")",sep=""),pt.lwd=2,cex=0.9,box.lty=0,pch=2,col="chartreuse3")

dev.off()
