
## remove (almost) everything in the working environment.
rm(list = ls())


#optimization package
if(!require(DEoptim)) {install.packages("DEoptim"); require(DEoptim)} 

# to import xl files
if(!require(readxl)) {install.packages("readxl"); require(readxl)} 


labels<- c("corr(F,S)","% S with 0","Q50 S","Q75 S","Q95 S","Mean S","% F with 0",
"Q50 F","Q75 F","Q95 F","Mean F","% S&F with 0","corr(GF,GS)")

setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\database")
#load database of families
familiesb <- as.data.frame(read_excel("Families-publis.xlsx"))

setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\estimation")
targets_se <- readRDS(file="targets_se.rds")

sink("sink-estimation by group.txt")

#initializing simulated moments
mom<-numeric(13)

#list of targets
targets<-numeric(13)

##############################parameters of the simulation ################
X<-50000  #number of artificial households
extra<-1000 # extra weight on the two extensive margins of publication
controlDE=list(itermax = 2000,trace = 25) # number of interations, 500 is not good enough

#to go fast
#X<-500  
#controlDE=list(itermax = 500,trace = 25)
###########################################################################


#initiating simulation
set.seed(5) #Use the set.seed function when running simulations to ensure all results, figures, etc are reproducible.
#draw four shocks from standardized normal
v<- (1:X)  # vector of probas.
y<-qnorm(v/(X+1))   # cumulative inverse normal
zz1 <-sample(y)    # to randomly reorganize the order of the numbers in a vector
zz2 <-sample(y)  
zz3 <-sample(y)
zz4 <-sample(y)
zz5 <-sample(y)   # distribution of ability shocks to grandsons' human capital
zz6 <-sample(y)

#define families
bbase <- data.frame("Fhum" = numeric(X), "ups"  = numeric(X), "Shum" = numeric(X),"GShum" = numeric(X))


# this function returns a dataframe with publications of both fathers and son (D for diads)
distribD1 <- function(params,base,z1,z2,z3,z4) { 
sigh<-params[1]
bet<-params[2]
sige<-params[3]
muh <-params[4]
muu <- (1-bet)*muh  # assumption of stagnation
sigu <- sqrt(1-bet^2)*sigh
mue<-0.
pthres <-params[5]
tau<-0;nu<-params[6]
    
base$Fhum <- muh+sigh*z1    # distribution of human capital of fathers 
base$Shum <- bet*base$Fhum +muu+sigu*z2 # human capital of sons

#selection of professors: tp (base of families father-sons) tp2 (base of families with 3 generations)
tp<-base[base$Fhum>tau & base$Shum>tau-nu,]

P<-nrow(tp)

#creating new data frame of selected professors
if (P<1){P<-1
tp<-base[1,]} #here we cheat: if nobody is selected, we still put the first guy in it to avoid a computational mistake. this outcome will never be selected anyhow

nbase <- data.frame("Fhum" = tp$Fhum,
                    "Shum" = tp$Shum,
                    "epsF" = mue+z3[1:P]*sige,  # shock to publications (fathers)  
                    "epsS" = mue+z4[1:P]*sige,  # shock to publications (sons)
                    "SPubli" = numeric(P),
                    "FPubli"= numeric(P))

nbase$SPubli <- ifelse(nbase$Shum+nbase$epsS<pthres,0,nbase$Shum+nbase$epsS) #publications
nbase$FPubli <- ifelse(nbase$Fhum+nbase$epsF<pthres,0,nbase$Fhum+nbase$epsF)

return(nbase)
}



# this function returns a dataframe with publications of both grand fathers and son (D for diads)
distribD2 <- function(params,base,z1,z2,z3,z4,z5,z6) { 
sigh<-params[1]
bet<-params[2]
sige<-params[3]
muh <-params[4]
muu <- (1-bet)*muh  # assumption of stagnation
sigu <- sqrt(1-bet^2)*sigh
mue<-0.
pthres <-params[5]
tau<-0;nu<-params[6]
   
base$Fhum <- muh+sigh*z1    # distribution of human capital of fathers 
base$Shum <- bet*base$Fhum +muu+sigu*z2  # human capital of sons
base$GShum <- bet*base$Shum +muu+sigu*z5  # human capital of grand sons

#selection of professors
tp<-base[base$GShum>tau-nu & base$Shum>tau-nu & base$Fhum>tau,]

P<-nrow(tp)

#creating new data frame of selected professors
if (P<1){P<-1
tp<-base[1,]} #here we cheat: if nobody is selected, we still put the first guy in it to avoid a computational mistake. this outcome will never be selected anyhow
nbase <- data.frame("Fhum" = tp$Fhum,
                    "Shum" = tp$Shum,
                    "GShum" = tp$GShum,
                    "epsF" = mue+z3[1:P]*sige,  # shock to publications (fathers)  
                    "epsS" = mue+z4[1:P]*sige,  # shock to publications (sons)
                    "epsGS" = mue+z6[1:P]*sige,  # shock to publications ( grand sons)
                    "SPubli" = numeric(P),
                    "FPubli"= numeric(P),
                    "GSPubli"= numeric(P))

nbase$GSPubli <- ifelse(nbase$GShum+nbase$epsGS<pthres,0,nbase$GShum+nbase$epsGS) #publications
nbase$SPubli <- ifelse(nbase$Shum+nbase$epsS<pthres,0,nbase$Shum+nbase$epsS) #publications
nbase$FPubli <- ifelse(nbase$Fhum+nbase$epsF<pthres,0,nbase$Fhum+nbase$epsF)

return(nbase)
}
objective <- function(pars,base,z1,z2,z3,z4,z5,z6,targ) { 
baseD<-distribD1(pars,base,z1,z2,z3,z4) #here we ask to compute the distributions resulting from parameters par
baseD2<-distribD2(pars,base,z1,z2,z3,z4,z5,z6) #here we ask to compute the distributions resulting from parameters par

PD <<- nrow(baseD)

#here we generate moments. First, correlation
mom[2] <- nrow(baseD[baseD$SPubli==0,])/PD #density at 0 for sons
mom[3]<- as.numeric(quantile(baseD$SPubli,0.5)) #median for sons
mom[4]<- as.numeric(quantile(baseD$SPubli,0.75)) # third quartile for sons
mom[5]<- as.numeric(quantile(baseD$SPubli,0.95)) #upper tail for sons
mom[6]<-mean(baseD$SPubli)
mom[7] <- nrow(baseD[baseD$FPubli==0,])/PD #same four moments for fathers
mom[8]<- as.numeric(quantile(baseD$FPubli,0.5))
mom[9]<- as.numeric(quantile(baseD$FPubli,0.75))
mom[10]<- as.numeric(quantile(baseD$FPubli,0.95))
mom[11]<-mean(baseD$FPubli)
mom[12]<- nrow(baseD[baseD$FPubli==0 & baseD$SPubli==0,])/PD
mom[1] <- ifelse(mom[2]==1 | mom[7]==1,0,cor(baseD[baseD$FPubli>0 & baseD$SPubli>0,"FPubli"],baseD[baseD$FPubli>0 & baseD$SPubli>0,"SPubli"]))
mom[13]<- ifelse(mom[2]==1 ,0,cor(baseD2[baseD2$FPubli>0 & baseD2$GSPubli>0,"FPubli"],baseD2[baseD2$FPubli>0 & baseD2$GSPubli>0,"GSPubli"]))
#base D2 is used only to compute mom[13]


mom0 <-extra*((mom[1]-targ[1])/targets_se[[1]])^2 +
        extra*((mom[2]-targ[2])/targets_se[[2]])^2 +
        ((mom[3]-targ[3])/targets_se[[3]])^2 + 
        ((mom[4]-targ[4])/targets_se[[4]])^2 + 
        ((mom[5]-targ[5])/targets_se[[5]])^2 +
        ((mom[6]-targ[6])/targets_se[[6]])^2 + 
        extra*((mom[7]-targ[7])/targets_se[[7]])^2 +
        ((mom[8]-targ[8])/targets_se[[8]])^2 +
        ((mom[9]-targ[9])/targets_se[[9]])^2 + 
        ((mom[10]-targ[10])/targets_se[[10]])^2 + 
        ((mom[11]-targ[11])/targets_se[[11]])^2 + 
        ((mom[12]-targ[12])/targets_se[[12]])^2 + 
        ifelse(!is.na(targ[13]),((mom[13]-targ[13])/targets_se[[13]])^2 ,0)



mom0<-ifelse(is.na(mom0),1000000,mom0)
outcome <<- mom       # the <<- operator assigns in the global environment
return(mom0)
}


#data frame of parameters & moments
par <- data.frame(sigh=numeric(),bet=numeric(),sige=numeric(),muh=numeric(),pthres=numeric(),nu=numeric())
targ <- data.frame(corrFS=numeric(),S0=numeric(),SQ50=numeric(),SQ75=numeric(),SQ95=numeric(),
Smean=numeric(),F0=numeric(),FQ50=numeric(),FQ75=numeric(),FQ95=numeric(),
Fmean=numeric(),SF0=numeric(),corrGFGS=numeric())


for (i in (1:23)){

#select sample########################

if (i==1){print("--------------------Families in Protestant universities (after 1527)--------------------")
families <-familiesb[familiesb$Protestant==1 & familiesb$Fdate>1527,]}
if (i==2){print("--------------------Families in Protestant universities without theology (after 1527)--------------------")
families <-familiesb[familiesb$Protestant==1 & familiesb$Fdate>1527  & familiesb$Field1!=1 & (is.na(familiesb$Field2) | familiesb$Field2!=1) &
     (is.na(familiesb$Field3) | familiesb$Field3!=1),]}
if (i==3){print("--------------------Families in Catholic universities (after 1527)--------------------")
families <-familiesb[familiesb$Catholic==1 & familiesb$Fdate>1527,]}


if (i==4){print("--------------------Families where Father is Lawyer--------------------")
families <-familiesb[familiesb$Field1==2 | (!is.na(familiesb$Field2) & familiesb$Field2==2) |
     (!is.na(familiesb$Field3) & familiesb$Field3==2),]}
if (i==5){print("--------------------Families where Father is Physician--------------------")
families <-familiesb[familiesb$Field1==4 | (!is.na(familiesb$Field2) & familiesb$Field2==4) |
     (!is.na(familiesb$Field3) & familiesb$Field3==4),]}
if (i==6){print("--------------------Families where Father is Theologian--------------------")
families <-familiesb[familiesb$Field1==1 | (!is.na(familiesb$Field2) & familiesb$Field2==1) |
     (!is.na(familiesb$Field3) & familiesb$Field3==1),]}
if (i==7){print("--------------------Families where Father is Scientist--------------------")
families <-familiesb[familiesb$Field1==5 | (!is.na(familiesb$Field2) & familiesb$Field2==5) |
     (!is.na(familiesb$Field3) & familiesb$Field3==5),]}


if (i==8){print("--------------------Families before 1543--------------------")
families <-familiesb[familiesb$Fdate<1543,]}
if (i==9){print("--------------------Families from 1543 to 1632--------------------")
families <-familiesb[familiesb$Fdate>1542 & familiesb$Fdate<=1632,]}
if (i==10){print("--------------------Families from 1633 to 1687--------------------")
families <-familiesb[familiesb$Fdate>1633 & familiesb$Fdate<=1687,]}
if (i==11){print("--------------------Families from 1688 to 1800--------------------")
families <-familiesb[familiesb$Fdate>1687,]}

if (i==12){print("--------------------Families where son is nominated while father is dead--------------------")
families <-familiesb[!is.na(familiesb$Fdied) & !is.na(familiesb$Sstart) & familiesb$Fdied<=familiesb$Sstart,]}
if (i==13){print("--------------------Families where son is nominated while father is alive--------------------")
families <-familiesb[!is.na(familiesb$Fdied) & !is.na(familiesb$Sstart) & familiesb$Fdied>familiesb$Sstart,]}

if (i==14){print("--------------------Families in new universities--------------------")
families <-familiesb[familiesb$New==1 & familiesb$Fdate>1542,]}
if (i==15){print("--------------------Families in old universities--------------------")
families <-familiesb[familiesb$New==0 & familiesb$Fdate>1542,]}

if (i==16){print("--------------------Families in universities--------------------")
families <-familiesb[familiesb$University==1 & familiesb$Fdate>1542,]}
if (i==17){print("--------------------Families in academies--------------------")
families <-familiesb[familiesb$University==0 & familiesb$Fdate>1542,]}

if (i==18){print("--------------------Son in same field as father--------------------")
families <-familiesb[familiesb$SameField==1 ,]}
if (i==19){print("--------------------Son in different field than father--------------------")
families <-familiesb[familiesb$SameField==0,]}

if (i==20){print("--------------------institutions with Comprehensive coverage--------------------")
families<-familiesb[familiesb$CovComprehensive==T ,]}

if (i==21){print("--------------------institutions with Comprehensive and Broad coverage--------------------")
families<-familiesb[familiesb$CovComprehensive==T | familiesb$CovBroad==T ,]}

if (i==22){print("--------------------All families--------------------")
families<-familiesb}

if (i==23){print("--------------------families parity 1--------------------")

setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\database")
families1 <- as.data.frame(read_excel("fathers-one-child.xlsx"))
setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\estimation")
families<-familiesb[familiesb$Father %in% families1$Father,]}

print(c("Number of families ",nrow(families)))

#list of sons
list_sons <- unique(families$Son)

list_fathers <- families[families$Father %in% list_sons,"Father"]
list_grandsons <-  families[families$Father %in% list_fathers,"Son"]

gfamilies <- data.frame("GrandFather"=character(),"GrandSon"=character(),"GFpubli"=numeric(),"GSpubli"=numeric(), 
                                 "GFin"=numeric(),"GSin"=numeric(),stringsAsFactors=FALSE)

ngfamil<-length(list_grandsons)

print(c("Number of grandsons ",ngfamil))

for (v in (1:ngfamil)){
grdson <- list_grandsons[v]
gfamilies[v,"GrandSon"]<-grdson
gfamilies[v,"GSpubli"]<-unique(families[families$Son==grdson,"Spubli"])
gfamilies[v,"GSin"]<-unique(families[families$Son==grdson,"Sin"])
son <- unique(families[families$Son==grdson,"Father"])
grdfather <- unique(families[families$Son==son,"Father"])
gfamilies[v,"GrandFather"]<-grdfather
gfamilies[v,"GFpubli"]<-unique(families[families$Father==grdfather,"Fpubli"])
gfamilies[v,"GFin"]<-unique(families[families$Father==grdfather,"Fin"])
}

targets[1] <- cor(families[families$Fin>0 & families$Sin>0,"Fpubli"],families[families$Fin>0 & families$Sin>0,"Spubli"])
targets[2] <- nrow(families[families$Sin==0,])/nrow(families) #density at 0 for sons
targets[3]<- as.numeric(quantile(families$Spubli,0.5)) #median for sons
targets[4]<- as.numeric(quantile(families$Spubli,0.75)) # third quartile for sons
targets[5]<- as.numeric(quantile(families$Spubli,0.95)) #upper tail for sons
targets[6]<-mean(families$Spubli)
targets[7] <- nrow(families[!(families$Father %in% list_sons) & families$Fin==0,])/nrow(families[!(families$Father %in% list_sons),]) #same four moments for fathers
targets[8]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.5))
targets[9]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.75))
targets[10]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.95))
targets[11]<-mean(families[!(families$Father %in% list_sons),"Fpubli"])
targets[12]<- nrow(families[families$Fin==0 & families$Sin==0,])/nrow(families)
targets[13] <- cor(gfamilies[gfamilies$GFin>0 & gfamilies$GSin>0,"GFpubli"],
                      gfamilies[gfamilies$GFin>0 & gfamilies$GSin>0,"GSpubli"])

print(cbind(labels,round(targets,3)))

obj <- function(x) {objective(x,bbase,zz1,zz2,zz3,zz4,zz5,zz6,targets)}
set.seed(7)
best2 <- DEoptim(obj,lower=c(.1,.1,.1,-2,0,0),upper=c(6,1,4,7,4,10),control=controlDE)
vecsol<-best2[[1]]$bestmem
print(cbind(c("sigh","bet","sige","muh","pthres","nu"),round(vecsol,3)))

#measuring the importance of nepotism
objective(vecsol,bbase,zz1,zz2,zz3,zz4,zz5,zz6,targets)
PD3<-PD
base3 <- distribD1(vecsol,bbase,zz1,zz2,zz3,zz4)
vecsolalt <- c(best2[[1]]$bestmem[1:5],0) #new vector of parameters setting nepotism to zero
base3alt <- distribD1(vecsolalt,bbase,zz1,zz2,zz3,zz4)
objective(vecsolalt,bbase,zz1,zz2,zz3,zz4,zz5,zz6,targets)
print(paste("Sons who should not be there :",round(100*(1-PD/PD3),2),"%"))
print(paste("Gain in mean p if no nepotism :",round((mean(base3alt$SPubli)/mean(base3$SPubli)-1)*100,2),"%"))

print(" ")  #new lines
print(" ")
}

sink()


