
## remove (almost) everything in the working environment.
rm(list = ls())


#optimization package
require(DEoptim)
#to read files
require(readxl)

if (!require('foreach')) install.packages('foreach'); library('foreach')
if (!require('doParallel')) install.packages('doParallel'); library('doParallel')

setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\database")


#load database of families
familiesb <- as.data.frame(read_excel("Families-publis.xlsx"))
Ndata0<- nrow(familiesb)
familiesb <- familiesb[familiesb$Sinwiki>0 | familiesb$Finwiki>0 | familiesb$Sinwc>0 | familiesb$Finwc>0,]  # keep families with wiki or wc
Ndata<- nrow(familiesb)

gfamiliesb <- as.data.frame(read_excel("GFamilies-publis.xlsx"))
listfathers <- unique(familiesb$Father)

setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\base")

outsidersb <-as.data.frame(read_excel("Profs-wiki-all.xlsx"))
outsidersb <- outsidersb[!(outsidersb$PKey %in% familiesb$Father),]  # remove families from outsiders
outsidersb <- outsidersb[!(outsidersb$PKey %in% familiesb$Son),]
Odata0<-nrow(outsidersb)
outsidersb <- outsidersb[outsidersb$pagewiki==1 | outsidersb$pagewc==1,]           # keep outsiders with wiki
Odata<-nrow(outsidersb)

#select outsiders in the same decade IKEY than sons
profs <-as.data.frame(read_excel("Profs.xlsx"))  #we need profs to get institutions
profs<-profs[,c("PKey","IKey")]
profs<-profs[profs$PKey %in% outsidersb$PKey,]

total <- merge(profs,outsidersb,by="PKey")
outsidersb <- total

crit<-unique(paste(familiesb$IKey,"+",floor(familiesb$Sdate/10),sep=""))
nrow(outsidersb)
outsidersb <- outsidersb[paste(outsidersb$IKey,"+",floor(outsidersb$refdatel/10),sep="") %in% crit,]
nrow(outsidersb)

df<- subset(outsidersb, select = -c(IKey) )
outsidersb<-unique(df)
nrow(outsidersb)
Odata<-nrow(outsidersb)


outsidersb$nlib <- ifelse(outsidersb$pagewc==1,outsidersb$nlib,0)
outsidersb$Oin <-  as.numeric(outsidersb$pagewc) 
outsidersb$Opubli <- asinh(outsidersb$nlib)




#we add five moments for outsiders
labels<- c("corr(F,S)","% S with 0","Q50 S","Q75 S","Q95 S","Mean S","% F with 0",
"Q50 F","Q75 F","Q95 F","Mean F","% S&F with 0","corr(GF,GS)","% O with 0","Q50 O","Q75 O","Q95 O","Mean O")

setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\outsiders")
targets_se <- readRDS(file="targets_se.rds")
targets <- as.vector(readRDS(file="targets_mean.rds"))

#initializing simulated moments
mom<-numeric(18)
 	

##############################parameters of the simulation ################
X<-50000  #number of artificial households
extra<-1000 # extra weight on the two extensive margins of publication
controlDE=list(itermax = 500,trace = 25) # number of interations, 500 is not good enough
nboot<-200 #number of artificial samples
###########################################################################



#initiating simulation
set.seed(5) #Use the set.seed function when running simulations to ensure all results, figures, etc are reproducible.
#draw four shocks from standardized normal
v<- (1:X)  # vector of probas.
y<-qnorm(v/(X+1))   # cumulative inverse normal
zz1 <-sample(y)    # to randomly reorganize the order of the numbers in a vector
zz2 <-sample(y)  
zz3 <-sample(y)
zz4 <-sample(y)
zz5 <-sample(y)   # distribution of ability shocks to grandsons' human capital
zz6 <-sample(y)

#define families
bbase <- data.frame("Fhum" = numeric(X), "Shum" = numeric(X),"GShum" = numeric(X))
obase <- data.frame("Ohum" = numeric(X))


# this function returns a dataframe with publications of both fathers and son (D for diads)
distribD1 <- function(params,base,z1,z2,z3,z4) { 
sigh<-params[1]
bet<-params[2]
sige<-params[3]
muh <-params[4]
muu <- (1-bet)*muh  # assumption of stagnation
sigu <- sqrt(1-bet^2)*sigh
mue<-0.
pthres <-params[5]
tau<-0
nu<-params[6]
    
base$Fhum <- muh+sigh*z1    # distribution of human capital of fathers 
base$Shum <- bet*base$Fhum +muu+sigu*z2 # human capital of sons

#selection of professors: tp (base of families father-sons) tp2 (base of families with 3 generations) tp3 (base of outsiders)
tp<-base[base$Fhum>tau & base$Shum>tau-nu,]
P<-nrow(tp)

#creating new data frame of selected professors
if (P<1){P<-1
tp<-base[1,]} #here we cheat: if nobody is selected, we still put the first guy in it to avoid a computational mistake. this outcome will never be selected anyhow

nbase <- data.frame("Fhum" = tp$Fhum,
                    "Shum" = tp$Shum,
                    "epsF" = mue+z3[1:P]*sige,  # shock to publications (fathers)  
                    "epsS" = mue+z4[1:P]*sige,  # shock to publications (sons)
                    "SPubli" = numeric(P),
                    "FPubli"= numeric(P))

nbase$SPubli <- ifelse(nbase$Shum+nbase$epsS<pthres,0,nbase$Shum+nbase$epsS) #publications
nbase$FPubli <- ifelse(nbase$Fhum+nbase$epsF<pthres,0,nbase$Fhum+nbase$epsF)

return(nbase)
}

# this function returns a dataframe with publications of outsiders
distribD3 <- function(params,base3,z1,z2,z3,z4) { 
sigh<-params[1]
sige<-params[3]
muh <-params[4]
mue<-0.
pthres <-params[5]
tau<-0.
    
base3$Ohum <- muh+sigh*z1    # distribution of human capital of outsiders
tp3<-base3[base3$Ohum>tau,]    #selection of professors:  tp3 (base of outsiders)
P3<-length(tp3)

#creating new data frame of selected professors
if (P3<1){P3<-1
tp3<-base3[1,]} #here we cheat: if nobody is selected, we still put the first guy in it to avoid a computational mistake. this outcome will never be selected anyhow

nbase3 <- data.frame("Ohum" = tp3,
                    "epsO" = mue+z3[1:P3]*sige,  # shock to publications 
                    "OPubli" = numeric(P3))

nbase3$OPubli <- ifelse(nbase3$Ohum+nbase3$epsO<pthres,0,nbase3$Ohum+nbase3$epsO)

return(nbase3)
}

# this function returns a dataframe with publications of both grand fathers and son (D for diads)
distribD2 <- function(params,base,z1,z2,z3,z4,z5,z6) { 
sigh<-params[1]
bet<-params[2]
sige<-params[3]
muh <-params[4]
muu <- (1-bet)*muh  # assumption of stagnation
sigu <- sqrt(1-bet^2)*sigh
mue<-0.
pthres <-params[5]

tau<-0;nu<-params[6]
    
base$Fhum <- muh+sigh*z1    # distribution of human capital of fathers 
base$Shum <- bet*base$Fhum +muu+sigu*z2  # human capital of sons
base$GShum <- bet*base$Shum +muu+sigu*z5  # human capital of grand sons

tp<-base[base$GShum>tau-nu & base$Shum>tau-nu & base$Fhum>tau,]

P<-nrow(tp)

#creating new data frame of selected professors
if (P<1){P<-1
tp<-base[1,]} #here we cheat: if nobody is selected, we still put the first guy in it to avoid a computational mistake. this outcome will never be selected anyhow
nbase <- data.frame("Fhum" = tp$Fhum,
                    "Shum" = tp$Shum,
                    "GShum" = tp$GShum,
                    "epsF" = mue+z3[1:P]*sige,  # shock to publications (fathers)  
                    "epsS" = mue+z4[1:P]*sige,  # shock to publications (sons)
                    "epsGS" = mue+z6[1:P]*sige,  # shock to publications ( grand sons)
                    "SPubli" = numeric(P),
                    "FPubli"= numeric(P),
                    "GSPubli"= numeric(P))

nbase$GSPubli <- ifelse(nbase$GShum+nbase$epsGS<pthres,0,nbase$GShum+nbase$epsGS) #publications
nbase$SPubli <- ifelse(nbase$Shum+nbase$epsS<pthres,0,nbase$Shum+nbase$epsS) #publications
nbase$FPubli <- ifelse(nbase$Fhum+nbase$epsF<pthres,0,nbase$Fhum+nbase$epsF)

return(nbase)
}


#defining the objective function
objective <- function(pars,base,base3,z1,z2,z3,z4,z5,z6,targ) { 
baseD<-distribD1(pars,base,z1,z2,z3,z4) #here we ask to compute the distributions resulting from parameters par
baseD2<-distribD2(pars,base,z1,z2,z3,z4,z5,z6) #here we ask to compute the distributions resulting from parameters par
baseD3<-distribD3(pars,base3,z1,z2,z3,z4) #here we ask to compute the distributions resulting from parameters par

PD <<- nrow(baseD)
PD3 <<- nrow(baseD3)

#initializing simulated moments
mom<-numeric(18)
#here we generate moments. First, correlation
mom[2] <- nrow(baseD[baseD$SPubli==0,])/PD #density at 0 for sons
mom[3]<- as.numeric(quantile(baseD$SPubli,0.5)) #median for sons
mom[4]<- as.numeric(quantile(baseD$SPubli,0.75)) # third quartile for sons
mom[5]<- as.numeric(quantile(baseD$SPubli,0.95)) #upper tail for sons
mom[6]<-mean(baseD$SPubli)
mom[7] <- nrow(baseD[baseD$FPubli==0,])/PD #same five moments for fathers
mom[8]<- as.numeric(quantile(baseD$FPubli,0.5))
mom[9]<- as.numeric(quantile(baseD$FPubli,0.75))
mom[10]<- as.numeric(quantile(baseD$FPubli,0.95))
mom[11]<-mean(baseD$FPubli)
mom[12]<- nrow(baseD[baseD$FPubli==0 & baseD$SPubli==0,])/PD
mom[1] <- ifelse(mom[2]==1 | mom[7]==1,0,cor(baseD[baseD$FPubli>0 & baseD$SPubli>0,"FPubli"],baseD[baseD$FPubli>0 & baseD$SPubli>0,"SPubli"]))
mom[13]<- ifelse(mom[2]==1 ,0,cor(baseD2[baseD2$FPubli>0 & baseD2$GSPubli>0,"FPubli"],baseD2[baseD2$FPubli>0 & baseD2$GSPubli>0,"GSPubli"]))
#base D2 is used only to compute mom[13]

mom[14] <- nrow(baseD3[baseD3$OPubli==0,])/PD3 #same five moments for outsiders
mom[15]<- as.numeric(quantile(baseD3$OPubli,0.5))
mom[16]<- as.numeric(quantile(baseD3$OPubli,0.75))
mom[17]<- as.numeric(quantile(baseD3$OPubli,0.95))
mom[18]<-mean(baseD3$OPubli)


#now objective function
mom0 <- extra*((mom[1]-targ[1])/targets_se[[1]])^2 +
        ((mom[2]-targ[2])/targets_se[[2]])^2 +
        ((mom[3]-targ[3])/targets_se[[3]])^2 + 
        ((mom[4]-targ[4])/targets_se[[4]])^2 + 
        ((mom[5]-targ[5])/targets_se[[5]])^2 +
        ((mom[6]-targ[6])/targets_se[[6]])^2 + 
        ((mom[7]-targ[7])/targets_se[[7]])^2 +
        ((mom[8]-targ[8])/targets_se[[8]])^2 +
        ((mom[9]-targ[9])/targets_se[[9]])^2 + 
        ((mom[10]-targ[10])/targets_se[[10]])^2 + 
        ((mom[11]-targ[11])/targets_se[[11]])^2 + 
        ((mom[12]-targ[12])/targets_se[[12]])^2 + 
        ((mom[13]-targ[13])/targets_se[[13]])^2 +
        extra * ((mom[2]-targ[2])/targets_se[[2]])^2 +extra* ((mom[7]-targ[7])/targets_se[[7]])^2

mom0<-ifelse(is.na(mom0),1000000,mom0)
outcome <<- mom       # the <<- operator assigns in the global environment
return(mom0)
}


#data frame of parameters & moments
par <- data.frame(sigh=numeric(),bet=numeric(),sige=numeric(),muh=numeric(),pthres=numeric(),nu=numeric(),share=numeric(),
corrFS=numeric(),S0=numeric(),SQ50=numeric(),SQ75=numeric(),SQ95=numeric(),
Smean=numeric(),F0=numeric(),FQ50=numeric(),FQ75=numeric(),FQ95=numeric(),
Fmean=numeric(),SF0=numeric(),corrGFGS=numeric(),
O0=numeric(),OQ50=numeric(),OQ75=numeric(),OQ95=numeric(),Omean=numeric())


#setup parallel backend to use many processors
cores=detectCores()
cl <- makeCluster(cores[1]-1) #not to overload your computer
registerDoParallel(cl)

parset <- foreach(i=1:nboot, .combine=rbind,.packages=c("DEoptim")) %dopar% {


set.seed(NULL)  #  re-initialized using a random seed based on the system clock.
#this is the line that generates a bootstrapped artificial sample
families <- familiesb[sample(nrow(familiesb),replace=T),]
outsiders <- outsidersb[sample(nrow(outsidersb),replace=T),]

#list of sons
list_sons <- unique(families$Son)

list_fathers <- families[families$Father %in% list_sons,"Father"]
list_grandsons <-  families[families$Father %in% list_fathers,"Son"]

gfamilies <- data.frame("GrandFather"=character(),"GrandSon"=character(),"GFpubli"=numeric(),"GSpubli"=numeric(), 
                                 "GFin"=numeric(),"GSin"=numeric(),stringsAsFactors=FALSE)

ngfamil<-length(list_grandsons)

for (v in (1:ngfamil)){
grdson <- list_grandsons[v]
gfamilies[v,"GrandSon"]<-grdson
gfamilies[v,"GSpubli"]<-unique(families[families$Son==grdson,"Spubli"])
gfamilies[v,"GSin"]<-unique(families[families$Son==grdson,"Sin"])
son <- unique(families[families$Son==grdson,"Father"])
grdfather <- unique(families[families$Son==son,"Father"])
gfamilies[v,"GrandFather"]<-grdfather
gfamilies[v,"GFpubli"]<-unique(families[families$Father==grdfather,"Fpubli"])
gfamilies[v,"GFin"]<-unique(families[families$Father==grdfather,"Fin"])
}

#list of targets
targets<-numeric(18)
targets[1] <- cor(families[!(families$Father %in% list_sons) & families$Fin>0 & families$Sin>0,"Fpubli"],
              families[!(families$Father %in% list_sons) & families$Fin>0 & families$Sin>0,"Spubli"])
targets[2] <- nrow(families[families$Sin==0,])/Ndata #density at 0 for sons
targets[3]<- as.numeric(quantile(families$Spubli,0.5)) #median for sons
targets[4]<- as.numeric(quantile(families$Spubli,0.75)) # third quartile for sons
targets[5]<- as.numeric(quantile(families$Spubli,0.95)) #upper tail for sons
targets[6]<-mean(families$Spubli)
targets[7] <- nrow(families[!(families$Father %in% list_sons) & families$Fin==0,])/nrow(families[!(families$Father %in% list_sons),]) #same four moments for fathers
targets[8]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.5))
targets[9]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.75))
targets[10]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.95))
targets[11]<-mean(families[!(families$Father %in% list_sons),"Fpubli"])
targets[12]<- nrow(families[families$Fin==0 & families$Sin==0,])/Ndata
targets[13] <- cor(gfamilies[gfamilies$GFin>0 & gfamilies$GSin>0,"GFpubli"],
                      gfamilies[gfamilies$GFin>0 & gfamilies$GSin>0,"GSpubli"])
targets[14] <- nrow(outsiders[outsiders$Oin==0,])/Odata #density at 0 for sons
targets[15]<- as.numeric(quantile(outsiders$Opubli,0.5)) #median for sons
targets[16]<- as.numeric(quantile(outsiders$Opubli,0.75)) # third quartile for sons
targets[17]<- as.numeric(quantile(outsiders$Opubli,0.95)) #upper tail for sons
targets[18]<-mean(outsiders$Opubli)



obj <- function(x) {objective(x,bbase,obase,zz1,zz2,zz3,zz4,zz5,zz6,targets)}
set.seed(7)
best2 <- DEoptim(obj,lower=c(.1,.1,.1,-2,0,0),upper=c(6,1,4,7,4,10),control=controlDE)
vecsol<-best2[[1]]$bestmem
objective(vecsol,bbase,obase,zz1,zz2,zz3,zz4,zz5,zz6,targets)
PD3<-PD
vecsolalt <- c(best2[[1]]$bestmem[1:5],0) #new vector of parameters setting nepotism to zero
base3alt <- distribD1(vecsolalt,bbase,zz1,zz2,zz3,zz4)
objective(vecsolalt,bbase,obase,zz1,zz2,zz3,zz4,zz5,zz6,targets)


par[i,1:6]<-best2[[1]]$bestmem
par[i,7]<- 100*(1-PD/PD3)
par[i,8:25]<-targets
par[i,,drop=FALSE] 
}

### parallel stuff
stopCluster(cl)

sink("sink-estimation SE1.txt")

print(parset)

print(apply(parset,2,mean))
print(apply(parset,2,sd))

sink()





#defining the NEW objective function  #############################################################################################
objective <- function(pars,base,base3,z1,z2,z3,z4,z5,z6,targ) { 
baseD<-distribD1(pars,base,z1,z2,z3,z4) #here we ask to compute the distributions resulting from parameters par
baseD2<-distribD2(pars,base,z1,z2,z3,z4,z5,z6) #here we ask to compute the distributions resulting from parameters par
baseD3<-distribD3(pars,base3,z1,z2,z3,z4) #here we ask to compute the distributions resulting from parameters par

PD <<- nrow(baseD)
PD3 <<- nrow(baseD3)

#initializing simulated moments
mom<-numeric(18)
#here we generate moments. First, correlation
mom[2] <- nrow(baseD[baseD$SPubli==0,])/PD #density at 0 for sons
mom[3]<- as.numeric(quantile(baseD$SPubli,0.5)) #median for sons
mom[4]<- as.numeric(quantile(baseD$SPubli,0.75)) # third quartile for sons
mom[5]<- as.numeric(quantile(baseD$SPubli,0.95)) #upper tail for sons
mom[6]<-mean(baseD$SPubli)
mom[7] <- nrow(baseD[baseD$FPubli==0,])/PD #same five moments for fathers
mom[8]<- as.numeric(quantile(baseD$FPubli,0.5))
mom[9]<- as.numeric(quantile(baseD$FPubli,0.75))
mom[10]<- as.numeric(quantile(baseD$FPubli,0.95))
mom[11]<-mean(baseD$FPubli)
mom[12]<- nrow(baseD[baseD$FPubli==0 & baseD$SPubli==0,])/PD
mom[1] <- ifelse(mom[2]==1 | mom[7]==1,0,cor(baseD[baseD$FPubli>0 & baseD$SPubli>0,"FPubli"],baseD[baseD$FPubli>0 & baseD$SPubli>0,"SPubli"]))
mom[13]<- ifelse(mom[2]==1 ,0,cor(baseD2[baseD2$FPubli>0 & baseD2$GSPubli>0,"FPubli"],baseD2[baseD2$FPubli>0 & baseD2$GSPubli>0,"GSPubli"]))
#base D2 is used only to compute mom[13]

mom[14] <- nrow(baseD3[baseD3$OPubli==0,])/PD3 #same five moments for outsiders
mom[15]<- as.numeric(quantile(baseD3$OPubli,0.5))
mom[16]<- as.numeric(quantile(baseD3$OPubli,0.75))
mom[17]<- as.numeric(quantile(baseD3$OPubli,0.95))
mom[18]<-mean(baseD3$OPubli)


#now objective function
mom0 <- extra*((mom[1]-targ[1])/targets_se[[1]])^2 +
        ((mom[2]-targ[2])/targets_se[[2]])^2 +
        ((mom[3]-targ[3])/targets_se[[3]])^2 + 
        ((mom[4]-targ[4])/targets_se[[4]])^2 + 
        ((mom[5]-targ[5])/targets_se[[5]])^2 +
        ((mom[6]-targ[6])/targets_se[[6]])^2 + 
        ((mom[7]-targ[7])/targets_se[[7]])^2 +
        ((mom[8]-targ[8])/targets_se[[8]])^2 +
        ((mom[9]-targ[9])/targets_se[[9]])^2 + 
        ((mom[10]-targ[10])/targets_se[[10]])^2 + 
        ((mom[11]-targ[11])/targets_se[[11]])^2 + 
        ((mom[12]-targ[12])/targets_se[[12]])^2 + 
        ((mom[13]-targ[13])/targets_se[[13]])^2 +
        extra * ((mom[2]-targ[2])/targets_se[[2]])^2 +extra* ((mom[7]-targ[7])/targets_se[[7]])^2 +
        ((mom[14]-targ[14])/targets_se[[14]])^2 + 
        ((mom[15]-targ[15])/targets_se[[15]])^2 +
        ((mom[16]-targ[16])/targets_se[[16]])^2 + 
        ((mom[17]-targ[17])/targets_se[[17]])^2 +
        ((mom[18]-targ[18])/targets_se[[18]])^2 

mom0<-ifelse(is.na(mom0),1000000,mom0)
outcome <<- mom       # the <<- operator assigns in the global environment
return(mom0)
}



#setup parallel backend to use many processors
cores=detectCores()
cl <- makeCluster(cores[1]-1) #not to overload your computer
registerDoParallel(cl)

parset <- foreach(i=1:nboot, .combine=rbind,.packages=c("DEoptim")) %dopar% {


set.seed(NULL)  #  re-initialized using a random seed based on the system clock.
#this is the line that generates a bootstrapped artificial sample
families <- familiesb[sample(nrow(familiesb),replace=T),]
outsiders <- outsidersb[sample(nrow(outsidersb),replace=T),]

#list of sons
list_sons <- unique(families$Son)

list_fathers <- families[families$Father %in% list_sons,"Father"]
list_grandsons <-  families[families$Father %in% list_fathers,"Son"]

gfamilies <- data.frame("GrandFather"=character(),"GrandSon"=character(),"GFpubli"=numeric(),"GSpubli"=numeric(), 
                                 "GFin"=numeric(),"GSin"=numeric(),stringsAsFactors=FALSE)

ngfamil<-length(list_grandsons)

for (v in (1:ngfamil)){
grdson <- list_grandsons[v]
gfamilies[v,"GrandSon"]<-grdson
gfamilies[v,"GSpubli"]<-unique(families[families$Son==grdson,"Spubli"])
gfamilies[v,"GSin"]<-unique(families[families$Son==grdson,"Sin"])
son <- unique(families[families$Son==grdson,"Father"])
grdfather <- unique(families[families$Son==son,"Father"])
gfamilies[v,"GrandFather"]<-grdfather
gfamilies[v,"GFpubli"]<-unique(families[families$Father==grdfather,"Fpubli"])
gfamilies[v,"GFin"]<-unique(families[families$Father==grdfather,"Fin"])
}

#list of targets
targets<-numeric(18)
targets[1] <- cor(families[!(families$Father %in% list_sons) & families$Fin>0 & families$Sin>0,"Fpubli"],
              families[!(families$Father %in% list_sons) & families$Fin>0 & families$Sin>0,"Spubli"])
targets[2] <- nrow(families[families$Sin==0,])/Ndata #density at 0 for sons
targets[3]<- as.numeric(quantile(families$Spubli,0.5)) #median for sons
targets[4]<- as.numeric(quantile(families$Spubli,0.75)) # third quartile for sons
targets[5]<- as.numeric(quantile(families$Spubli,0.95)) #upper tail for sons
targets[6]<-mean(families$Spubli)
targets[7] <- nrow(families[!(families$Father %in% list_sons) & families$Fin==0,])/nrow(families[!(families$Father %in% list_sons),]) #same four moments for fathers
targets[8]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.5))
targets[9]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.75))
targets[10]<- as.numeric(quantile(families[!(families$Father %in% list_sons),"Fpubli"],0.95))
targets[11]<-mean(families[!(families$Father %in% list_sons),"Fpubli"])
targets[12]<- nrow(families[families$Fin==0 & families$Sin==0,])/Ndata
targets[13] <- cor(gfamilies[gfamilies$GFin>0 & gfamilies$GSin>0,"GFpubli"],
                      gfamilies[gfamilies$GFin>0 & gfamilies$GSin>0,"GSpubli"])
targets[14] <- nrow(outsiders[outsiders$Oin==0,])/Odata #density at 0 for sons
targets[15]<- as.numeric(quantile(outsiders$Opubli,0.5)) #median for sons
targets[16]<- as.numeric(quantile(outsiders$Opubli,0.75)) # third quartile for sons
targets[17]<- as.numeric(quantile(outsiders$Opubli,0.95)) #upper tail for sons
targets[18]<-mean(outsiders$Opubli)


obj <- function(x) {objective(x,bbase,obase,zz1,zz2,zz3,zz4,zz5,zz6,targets)}
set.seed(7)
best2 <- DEoptim(obj,lower=c(.1,.1,.1,-2,0,0),upper=c(6,1,4,7,4,10),control=controlDE)

vecsol<-best2[[1]]$bestmem
objective(vecsol,bbase,obase,zz1,zz2,zz3,zz4,zz5,zz6,targets)
PD3<-PD
vecsolalt <- c(best2[[1]]$bestmem[1:5],0) #new vector of parameters setting nepotism to zero
base3alt <- distribD1(vecsolalt,bbase,zz1,zz2,zz3,zz4)
objective(vecsolalt,bbase,obase,zz1,zz2,zz3,zz4,zz5,zz6,targets)


par[i,1:6]<-best2[[1]]$bestmem
par[i,7]<- 100*(1-PD/PD3)
par[i,8:25]<-targets

par[i,,drop=FALSE] 
}

### parallel stuff
stopCluster(cl)

sink("sink-estimation SE2.txt")

print(parset)

print(apply(parset,2,mean))
print(apply(parset,2,sd))

sink()


