
## remove (almost) everything in the working environment.
rm(list = ls())


#optimization package
require(DEoptim)
#to read files
require(readxl)



labels<- c("corr(F,S)","% S with 0","Q50 S","Q75 S","Q95 S","Mean S","% F with 0",
"Q50 F","Q75 F","Q95 F","Mean F","% S&F with 0","corr(GF,GS)")

setwd("C:\\Users\\ddlc\\Documents\\Dropbox\\01. Research\\UTHC social mobility\\robustness")
targets_se <- readRDS(file="targets_se_nworks.rds")
targets <- as.vector(readRDS(file="targets_mean_nworks.rds"))

sink("sink-estimation robust nworks.txt")

#initializing simulated moments
mom<-numeric(13)
 	

##############################parameters of the simulation ################
X<-50000  #number of artificial households
extra<-1000 # extra weight on the two extensive margins of publication
controlDE=list(itermax = 2000,trace = 25) # number of interations, 500 is not good enough
###########################################################################



#initiating simulation
set.seed(5) #Use the set.seed function when running simulations to ensure all results, figures, etc are reproducible.
#draw four shocks from standardized normal
v<- (1:X)  # vector of probas.
y<-qnorm(v/(X+1))   # cumulative inverse normal
zz1 <-sample(y)    # to randomly reorganize the order of the numbers in a vector
zz2 <-sample(y)  
zz3 <-sample(y)
zz4 <-sample(y)
zz5 <-sample(y)   # distribution of ability shocks to grandsons' human capital
zz6 <-sample(y)

#define families
bbase <- data.frame("Fhum" = numeric(X), "Shum" = numeric(X),"GShum" = numeric(X))


###############################################################################
# The models
# 1. no selection (5 parameters)
# 2. with selection (5 parameters)
# 3. with selection and nepotism (6 parameters)
###############################################################################


# this function returns a dataframe with publications of both fathers and son (D for diads)
distribD1 <- function(mod,params,base,z1,z2,z3,z4) { 
sigh<-params[1]
bet<-params[2]
sige<-params[3]
muh <-params[4]
muu <- (1-bet)*muh  # assumption of stagnation
sigu <- sqrt(1-bet^2)*sigh
mue<-0.
pthres <-params[5]

if (mod==2){tau<-0}   #selection parameters
if (mod==3){tau<-0;nu<-params[6]}
    
base$Fhum <- muh+sigh*z1    # distribution of human capital of fathers 
base$Shum <- bet*base$Fhum +muu+sigu*z2 # human capital of sons

#selection of professors: tp (base of families father-sons) tp2 (base of families with 3 generations)
if (mod==1){tp<-base}
if (mod==2){tp<-base[base$Fhum>tau & base$Shum>tau,]}
if (mod==3){tp<-base[base$Fhum>tau & base$Shum>tau-nu,]}

P<-nrow(tp)

#creating new data frame of selected professors
if (P<1){P<-1
tp<-base[1,]} #here we cheat: if nobody is selected, we still put the first guy in it to avoid a computational mistake. this outcome will never be selected anyhow

nbase <- data.frame("Fhum" = tp$Fhum,
                    "Shum" = tp$Shum,
                    "epsF" = mue+z3[1:P]*sige,  # shock to publications (fathers)  
                    "epsS" = mue+z4[1:P]*sige,  # shock to publications (sons)
                    "SPubli" = numeric(P),
                    "FPubli"= numeric(P))

nbase$SPubli <- ifelse(nbase$Shum+nbase$epsS<pthres,0,nbase$Shum+nbase$epsS) #publications
nbase$FPubli <- ifelse(nbase$Fhum+nbase$epsF<pthres,0,nbase$Fhum+nbase$epsF)

return(nbase)
}



# this function returns a dataframe with publications of both grand fathers and son (D for diads)
distribD2 <- function(mod,params,base,z1,z2,z3,z4,z5,z6) { 
sigh<-params[1]
bet<-params[2]
sige<-params[3]
muh <-params[4]
muu <- (1-bet)*muh  # assumption of stagnation
sigu <- sqrt(1-bet^2)*sigh
mue<-0.
pthres <-params[5]

if (mod==2){tau<-0}   #selection parameters
if (mod==3){tau<-0;nu<-params[6]}
    
base$Fhum <- muh+sigh*z1    # distribution of human capital of fathers 
base$Shum <- bet*base$Fhum +muu+sigu*z2  # human capital of sons
base$GShum <- bet*base$Shum +muu+sigu*z5  # human capital of grand sons

#selection of professors
if (mod==1){tp<-base}
if (mod==2){tp<-base[base$Shum>tau & base$Fhum>tau & base$GShum>tau,]}
if (mod==3){tp<-base[base$GShum>tau-nu & base$Shum>tau-nu & base$Fhum>tau,]}

P<-nrow(tp)

#creating new data frame of selected professors
if (P<1){P<-1
tp<-base[1,]} #here we cheat: if nobody is selected, we still put the first guy in it to avoid a computational mistake. this outcome will never be selected anyhow
nbase <- data.frame("Fhum" = tp$Fhum,
                    "Shum" = tp$Shum,
                    "GShum" = tp$GShum,
                    "epsF" = mue+z3[1:P]*sige,  # shock to publications (fathers)  
                    "epsS" = mue+z4[1:P]*sige,  # shock to publications (sons)
                    "epsGS" = mue+z6[1:P]*sige,  # shock to publications ( grand sons)
                    "SPubli" = numeric(P),
                    "FPubli"= numeric(P),
                    "GSPubli"= numeric(P))

nbase$GSPubli <- ifelse(nbase$GShum+nbase$epsGS<pthres,0,nbase$GShum+nbase$epsGS) #publications
nbase$SPubli <- ifelse(nbase$Shum+nbase$epsS<pthres,0,nbase$Shum+nbase$epsS) #publications
nbase$FPubli <- ifelse(nbase$Fhum+nbase$epsF<pthres,0,nbase$Fhum+nbase$epsF)

return(nbase)
}


#defining the objective function
objective <- function(mod,pars,base,z1,z2,z3,z4,z5,z6,targ) { 
baseD<-distribD1(mod,pars,base,z1,z2,z3,z4) #here we ask to compute the distributions resulting from parameters par
baseD2<-distribD2(mod,pars,base,z1,z2,z3,z4,z5,z6) #here we ask to compute the distributions resulting from parameters par

PD <<- nrow(baseD)

#here we generate moments. First, correlation
mom[2] <- nrow(baseD[baseD$SPubli==0,])/PD #density at 0 for sons
mom[3]<- as.numeric(quantile(baseD$SPubli,0.5)) #median for sons
mom[4]<- as.numeric(quantile(baseD$SPubli,0.75)) # third quartile for sons
mom[5]<- as.numeric(quantile(baseD$SPubli,0.95)) #upper tail for sons
mom[6]<-mean(baseD$SPubli)
mom[7] <- nrow(baseD[baseD$FPubli==0,])/PD #same four moments for fathers
mom[8]<- as.numeric(quantile(baseD$FPubli,0.5))
mom[9]<- as.numeric(quantile(baseD$FPubli,0.75))
mom[10]<- as.numeric(quantile(baseD$FPubli,0.95))
mom[11]<-mean(baseD$FPubli)
mom[12]<- nrow(baseD[baseD$FPubli==0 & baseD$SPubli==0,])/PD
mom[1] <- ifelse(mom[2]==1 | mom[7]==1,0,cor(baseD[baseD$FPubli>0 & baseD$SPubli>0,"FPubli"],baseD[baseD$FPubli>0 & baseD$SPubli>0,"SPubli"]))
mom[13]<- ifelse(mom[2]==1 ,0,cor(baseD2[baseD2$FPubli>0 & baseD2$GSPubli>0,"FPubli"],baseD2[baseD2$FPubli>0 & baseD2$GSPubli>0,"GSPubli"]))
#base D2 is used only to compute mom[13]

#now objective function
mom0 <- extra*((mom[1]-targ[1])/targets_se[[1]])^2 +
        ((mom[2]-targ[2])/targets_se[[2]])^2 +
        ((mom[3]-targ[3])/targets_se[[3]])^2 + 
        ((mom[4]-targ[4])/targets_se[[4]])^2 + 
        ((mom[5]-targ[5])/targets_se[[5]])^2 +
        ((mom[6]-targ[6])/targets_se[[6]])^2 + 
        ((mom[7]-targ[7])/targets_se[[7]])^2 +
        ((mom[8]-targ[8])/targets_se[[8]])^2 +
        ((mom[9]-targ[9])/targets_se[[9]])^2 + 
        ((mom[10]-targ[10])/targets_se[[10]])^2 + 
        ((mom[11]-targ[11])/targets_se[[11]])^2 + 
        ((mom[12]-targ[12])/targets_se[[12]])^2 + 
        ((mom[13]-targ[13])/targets_se[[13]])^2 +
        ifelse(mod==3,extra * ((mom[2]-targ[2])/targets_se[[2]])^2 +extra* ((mom[7]-targ[7])/targets_se[[7]])^2,0)


mom0<-ifelse(is.na(mom0),1000000,mom0)
outcome <<- mom       # the <<- operator assigns in the global environment
return(mom0)
}

objective(1,c(5,1,3,5,4),bbase,zz1,zz2,zz3,zz4,zz5,zz6,targets)



# model 3

obj <- function(x) {objective(3,x,bbase,zz1,zz2,zz3,zz4,zz5,zz6,targets)}
set.seed(7)
best2 <- DEoptim(obj,lower=c(.1,.1,.1,-2,0,0),upper=c(6,1,4,7,4,10),control=controlDE)
vecsol<-best2[[1]]$bestmem
objective(3,vecsol,bbase,zz1,zz2,zz3,zz4,zz5,zz6,targets)
print(cbind(c("sigh","bet","sige","muh","pthres","nu"),round(vecsol,3)))
print(cbind(labels,round(targets,3),round(outcome,3)))

PD3<-PD
base3 <- distribD1(3,vecsol,bbase,zz1,zz2,zz3,zz4)



#measuring the importance of nepotism
vecsolalt <- c(best2[[1]]$bestmem[1:5],0) #new vector of parameters setting nepotism to zero
base3alt <- distribD1(3,vecsolalt,bbase,zz1,zz2,zz3,zz4)
objective(3,vecsolalt,bbase,zz1,zz2,zz3,zz4,zz5,zz6,targets)


print(paste("Sons who should not be there :",round(100*(1-PD/PD3),2),"%"))
print(paste("Gain in mean p if no nepotism :",round((mean(base3alt$SPubli)/mean(base3$SPubli)-1)*100,2),"%"))

sink()

